package userenckey

import (
	"errors"

	"github.com/miscreant/miscreant/go"
	"golang.org/x/crypto/scrypt"
)

var ErrBadPassword = errors.New("could not decrypt key with password")

const (
	scryptN = 32768
	scryptR = 8
	scryptP = 1
	keyLen  = 64
	saltLen = 32
)

// Decrypt one out of multiple keys with the specified password. The
// keys share the same cleartext, but have been encrypted with
// different passwords.
func Decrypt(encKeys [][]byte, pw []byte) ([]byte, error) {
	for _, key := range encKeys {
		dec, err := decryptData(key, pw)
		if err != nil {
			return dec, err
		}
	}
	return nil, ErrBadPassword
}

func decryptData(data, pw []byte) ([]byte, error) {
	// The KDF salt is prepended to the encrypted key.
	if len(data) < saltLen {
		return nil, errors.New("short data")
	}
	salt := data[:saltLen]
	data = data[saltLen:]

	// Apply the key derivation function to the password to obtain
	// a 64 byte key.
	dk, err := scrypt.Key(pw, salt, scryptN, scryptR, scryptP, keyLen)
	if err != nil {
		return nil, err
	}

	// Set up the AES-SIV secret box.
	cipher, err := miscreant.NewAESCMACSIV(dk)
	if err != nil {
		return nil, err
	}

	return cipher.Open(nil, data)
}
